package com.xhkjedu.sclass.service.classroom;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.sclass.mapper.classroom.ClassroomPaperFileMapper;
import com.xhkjedu.sclass.mapper.classroom.ClassroomPaperMapper;
import com.xhkjedu.sclass.mapper.classroom.ClassroomPaperQtypeMapper;
import com.xhkjedu.sclass.mapper.classroom.ClassroomPaperScantronMapper;
import com.xhkjedu.sclass.mapper.classroom.ClassroomPaperStudentMapper;
import com.xhkjedu.sclass.mapper.classroom.ClassroomPaperStudentScantronMapper;
import com.xhkjedu.sclass.model.classroom.TClassroomPaper;
import com.xhkjedu.sclass.model.classroom.TClassroomPaperFile;
import com.xhkjedu.sclass.model.classroom.TClassroomPaperQtype;
import com.xhkjedu.sclass.model.classroom.TClassroomPaperScantron;
import com.xhkjedu.sclass.model.classroom.TClassroomPaperStudent;
import com.xhkjedu.sclass.model.classroom.TClassroomPaperStudentScantron;
import com.xhkjedu.sclass.model.paperstudent.TPaperStudentScantron;
import com.xhkjedu.utils.N_Utils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author ywx
 * @classname RoomPaperService
 * @description 堂测
 * @date 2020/7/15 8:20
 **/
@Slf4j
@Service
public class ClassroomPaperService {
    @Resource
    private ClassroomPaperMapper classroomPaperMapper;
    @Resource
    private ClassroomPaperFileMapper classroomPaperFileMapper;
    @Resource
    private ClassroomPaperQtypeMapper classroomPaperQtypeMapper;
    @Resource
    private ClassroomPaperScantronMapper classroomPaperScantronMapper;
    @Resource
    private ClassroomPaperStudentMapper classroomPaperStudentMapper;
    @Resource
    private ClassroomPaperStudentScantronMapper classroomPaperStudentScantronMapper;

    /**
     * 答题卡作业--保存
     * @Param [paper]
     * @Author ywx
     * @Date 2020/7/15 8:22
     * @return void
     **/
    @Transactional(rollbackFor = Exception.class)
    public void saveScantronPaper(TClassroomPaper paper) {
        try {
            List<TClassroomPaperQtype> qtypes = paper.getQtypes();
            //答题卡中试题
            List<TClassroomPaperScantron> scanlist = new ArrayList<>();
            qtypes.forEach(t->{
                scanlist.addAll(t.getScantrons());
            });
            paper.setPapernum(scanlist.size());
            Double paperscore = scanlist.stream().collect(Collectors.summingDouble(TClassroomPaperScantron::getPsscore));
            paper.setPaperscore(paperscore);
            String directorid = paper.getDirectorid();
            if (directorid != null && (directorid.equals("0") || directorid.equals(""))) {
                paper.setDirectorid(null);
            }
            classroomPaperMapper.insertSelective(paper);//保存作业

            //保存作业关联信息
            List<TClassroomPaperFile> lst = paper.getPfiles();
            if(lst==null || lst.size()==0 || scanlist ==null || scanlist.size()==0){
            }else{
                //保存作业附件
                savePaperScantronFile(paper);
                //保存答题卡信息
                savePaperScantronQtype(paper,qtypes);
            }

        }catch (Exception e){
            log.error("保存答题卡作业失败：" + e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
    }

    //课堂考试--保存附件信息
    private void savePaperScantronFile(TClassroomPaper paper){
        List<TClassroomPaperFile> lst = paper.getPfiles();
        for(TClassroomPaperFile pf : lst){
            pf.setCreateid(paper.getCreateid());
            pf.setCreatetime(N_Utils.getSecondTimestamp());
            pf.setSchoolid(paper.getSchoolid());
            pf.setPaperid(paper.getPaperid());
        }
        classroomPaperFileMapper.insertList(lst);
    }

    //课堂考试--保存答题卡-题型-试题
    private void savePaperScantronQtype(TClassroomPaper paper,List<TClassroomPaperQtype> qtypes){
        int sorder = 1;//答题卡试题序号
        int qtypeorder = 1;//答题卡题型序号
        //处理答题卡--题型--试题
        List<TClassroomPaperScantron> savescan = new ArrayList<>();
        for(TClassroomPaperQtype qtype : qtypes){
            List<TClassroomPaperScantron> scanlst = qtype.getScantrons();//题型下试题
            qtype.setCreateid(paper.getCreateid());
            qtype.setCreatetime(paper.getCreatetime());
            qtype.setSchoolid(paper.getSchoolid());
            qtype.setPqnum(scanlst.size());
            Double qtypescore = scanlst.stream().collect(Collectors.summingDouble(TClassroomPaperScantron::getPsscore));
            qtype.setPqscore(qtypescore);
            qtype.setPaperid(paper.getPaperid());
            qtype.setPqorder(qtypeorder);
            qtypeorder++;

            for(TClassroomPaperScantron sc : scanlst){
                sc.setCreateid(paper.getCreateid());
                sc.setCreatetime(N_Utils.getSecondTimestamp());
                sc.setSchoolid(paper.getSchoolid());
                sc.setPaperid(paper.getPaperid());
                sc.setPsorder(sorder);
                savescan.add(sc);
                sorder++;
            }
        }

        //保存答题卡题型
        classroomPaperQtypeMapper.batchSave(qtypes);
        //保存答题卡题型下试题
        classroomPaperScantronMapper.batchSave(savescan);
    }

    /**
     * 堂测--学生作答详情
     * @Param [scantrons]
     * @Author ywx
     * @Date 2020/7/15 9:34
     * @return void
     **/
    @Transactional(rollbackFor = Exception.class)
    public void saveStudentScantron(TClassroomPaperStudent student) {
        try {
            classroomPaperStudentMapper.insertUseGeneratedKeys(student);
            Integer psid = student.getId();
            List<TClassroomPaperStudentScantron> scantrons = student.getScantrons();
            int qorder = 1;//试题序号
            for (TClassroomPaperStudentScantron s : scantrons) {
                s.setPsid(psid);
                s.setQorder(qorder);
                qorder++;
            }
            classroomPaperStudentScantronMapper.insertList(scantrons);
        } catch (Exception e) {
            log.error("保存学生作答详情失败："+e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public int correctExam(Integer psid, Integer checkid, String questionResult) throws Exception {
        try {
            TClassroomPaperStudent ps = classroomPaperStudentMapper.findById(psid);
            Long paperid = ps.getPaperid();
            Integer classid = ps.getClassid();
            List<TPaperStudentScantron> psslist = JSON.parseArray(questionResult, TPaperStudentScantron.class);
            if (N_Utils.isListNotEmpty(psslist)) {
                for (TPaperStudentScantron pss : psslist) {
                    //设置批改状态为已批改
                    pss.setCheckid(checkid);
                    pss.setChecked(1);
                    pss.setChecktime(N_Utils.getSecondTimestamp());
                }

                //批量更新试题批改信息
                classroomPaperStudentScantronMapper.updateScantronQuestionBatch(psslist);
            }
            //更新用户得分
            classroomPaperStudentMapper.updateScantronStuScore(psid);
            //判断该作业该班级时候已经批改完成
            //查询如果为0，则证明本作业本班级截止当前时间已经提交作业学生全部批改完成
            Integer notCorrect = classroomPaperStudentMapper.getNotCorrect(paperid, classid);
            return notCorrect;
        } catch (Exception e) {
            log.error("课堂考试批阅失败："+e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            throw new Exception("课堂考试批阅失败");
        }
    }

    /**
     * 根据考试学生id获取考试是否完成批阅
     * @Param [psid]
     * @Author ywx
     * @Date 2020/9/10 16:23
     * @return java.lang.Integer
     **/
    public Integer getCheckedByPsId(Integer psid) {
        return classroomPaperStudentMapper.getCheckedByPsId(psid);
    }

    /**
     * 获取考试学生待批改试题列表
     * @Param [psid]
     * @Author ywx
     * @Date 2020/9/10 16:24
     * @return java.util.List<com.xhkjedu.vo.paperstudent.PaperStudentScantronVo>
     **/
    public List<Map> listCorrectQuestion(Integer psid) {
        return classroomPaperStudentScantronMapper.listCorrectQuestion(psid);
    }

    /**
     * 修改学生作答记录（教师添加批注）
     * @Param [pssid, useranswer]
     * @Author ywx
     * @Date 2020/9/11 15:15
     * @return void
     **/
    public void updateScantronUseranswer(Integer pssid, String useranswer) {
        classroomPaperStudentScantronMapper.updateScantronUseranswer(pssid, useranswer);
    }

    /**
     * 考试按人统计
     * @Param [paperid]
     * @Author ywx
     * @Date 2022/3/11 16:48
     * @return java.util.List<java.util.Map>
     **/
    public List<Map> listEStudent(Long paperid) {
        return classroomPaperStudentMapper.listEStudent(paperid);
    }
}
