package com.xhkjedu.suser.mapper.system;

import com.xhkjedu.suser.base.TkMapper;
import com.xhkjedu.suser.model.system.TUser;
import com.xhkjedu.suser.vo.assess.AssessParams;
import com.xhkjedu.suser.vo.system.TeacherVo;
import com.xhkjedu.suser.vo.system.UserVo;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @ClassName UserMapper
 * Description
 * Author WN
 * Date 2020/5/29 17:00
 **/
@Repository
public interface UserMapper extends TkMapper<TUser> {

    /**
     *功能描述  根据帐号密码获取用户
     * @author WN
     * @date 2020/5/29
     * @param  * @param loginname
     * @return com.xhkjedu.model.system.TUser
     */
    UserVo getUserByLoginname(String loginname);

    /**
     * 删除用户
     * @Param [user]
     * @Author ywx
     * @Date 2020/6/2 9:59
     * @return java.lang.Integer
     **/
    Integer deleteById(@Param("user") TUser user);

    /**
     * 获取用户列表
     * @Param [user]
     * @Author ywx
     * @Date 2020/6/2 10:07
     * @return java.util.List<com.xhkjedu.suser.vo.system.UserVo>
     **/
    List<UserVo> findAll(@Param("user") TUser user);

    /**
     * 获取用户详情
     * @Param [userid]
     * @Author ywx
     * @Date 2020/6/2 10:23
     * @return com.xhkjedu.suser.vo.system.UserVo
     **/
    UserVo findById(@Param("userid") Integer userid);

    /**
     * 修改用户密码
     * @Param [user]
     * @Author ywx
     * @Date 2020/6/2 10:30
     * @return java.lang.Integer
     **/
    Integer updatePwd(@Param("user") TUser user);

    /**
     * 修改用户头像
     * @Param [user]
     * @Author ywx
     * @Date 2020/6/2 10:36
     * @return java.lang.Integer
     **/
    Integer updateHeadPic(@Param("user") TUser user);

    /**
     * 检查登录账号是否存在
     * @Param [loginname]
     * @Author ywx
     * @Date 2020/6/2 10:44
     * @return java.lang.Integer
     **/
    Integer checkLoginName(@Param("loginname") String loginname);

    /**
     * 根据登录账号获取用户详情
     * @Param [loginname]
     * @Author ywx
     * @Date 2020/6/3 9:33
     * @return com.xhkjedu.model.system.TUser
     **/
    TUser getObjByLoginName(@Param("loginname") String loginname);

    /**
     * 修改用户状态
     * @Param [userid, userstate]
     * @Author ywx
     * @Date 2020/6/4 9:08
     * @return void
     **/
    void updateState(@Param("userid") Integer userid,@Param("userstate") int userstate);

    /**
     *功能描述 设置操作密码
     * @author WN
     * @date 2020/6/17
     * @param  * @param userid
     * @param opepassword
     * @return void
     */
    void updateOpepasswordByUserId(@Param("userid") Integer userid,@Param("opepassword") String opepassword);

    /**
     *功能描述 验证用户操作密码是否正确
     * @author WN
     * @date 2020/6/17
     * @param  * @param userid
     * @param opepassword
     * @return java.lang.Integer
     */
    Integer getUserIdByOpepassword(@Param("userid") Integer userid,@Param("opepassword") String opepassword);

    /**
     * 教师基本信息
     * @Param [userid]
     * @Author ywx
     * @Date 2020/6/29 16:24
     * @return com.xhkjedu.suser.vo.system.TeacherVo
     **/
    TeacherVo getTeacherInfo(Integer userid, Integer classtype);

    /**
     * 更新学生硬件绑定信息
     * @Param [user]
     * @Author ywx
     * @Date 2020/9/18 11:38
     * @return void
     **/
    void updateBindMac(@Param("user") UserVo user);

    /**
     * 根据登录账号和硬件地址获取用户信息
     * @Param [loginname, mac]
     * @Author ywx
     * @Date 2020/9/21 16:17
     * @return com.xhkjedu.suser.vo.system.UserVo
     **/
    UserVo getUserByLoginnameMac(String loginname, String mac);

    /**
     * 通过短信修改密码
     * @Param [user]
     * @Author ywx
     * @Date 2020/10/13 15:17
     * @return void
     **/
    void updatePwdByCode(@Param("user") TUser user);

    /**
     * 批量修改用户密码
     * @Param [user]
     * @Author ywx
     * @Date 2020/10/30 16:04
     * @return void
     **/
    void batchUpdatePwd(@Param("user") TUser user);

    //校验手机号
    boolean checkUserPhone(Integer userid);

    //获取学校账号数量
    Integer getUserNumBySchoolId(Integer schoolid);

    //用户选择列表
    List<Map> listForSel(@Param("user") AssessParams user, @Param("year") Integer year);

    //获取修改头像时间
    Integer getHeadTime(Integer userid);
}
