package com.xhkjedu.suser.service.system;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.xhkjedu.suser.config.ConfigKey;
import com.xhkjedu.suser.mapper.gradeclass.SchoolMapper;
import com.xhkjedu.suser.mapper.system.MsgMapper;
import com.xhkjedu.suser.mapper.system.UserLogMapper;
import com.xhkjedu.suser.mapper.system.UserMapper;
import com.xhkjedu.suser.model.system.TMsg;
import com.xhkjedu.suser.model.system.TUser;
import com.xhkjedu.suser.model.system.TUserLog;
import com.xhkjedu.suser.utils.JedisUtil;
import com.xhkjedu.suser.utils.PoiUtils;
import com.xhkjedu.suser.utils.SMSSender;
import com.xhkjedu.suser.vo.assess.AssessParams;
import com.xhkjedu.suser.vo.system.TeacherVo;
import com.xhkjedu.suser.vo.system.UserVo;
import com.xhkjedu.suser.vo.system.ZtMsgVo;
import com.xhkjedu.utils.DESUtils;
import com.xhkjedu.utils.DatesUtil;
import com.xhkjedu.utils.MD5;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.utils.PageUtil;
import com.xhkjedu.vo.PageResult;
import com.xhkjedu.vo.ResultVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * @ClassName UserService
 * Description
 * Author WN
 * Date 2020/5/29 16:31
 **/
@Service
@Slf4j
public class UserService extends JedisUtil {

    @Autowired
    private UserMapper userMapper;
    @Autowired
    private UserLogMapper userLogMapper;
    @Resource
    private MsgMapper msgMapper;
    @Resource
    private SchoolMapper schoolMapper;

    /**
     *功能描述  根据登录名和密码获取用户信息
     * @author WN
     * @date 2020/6/1
     * @param  * @param user
     * @return com.xhkjedu.vo.ResultVo
     */
    public ResultVo getUserByLoginnameAndPwd(TUser user,HttpServletRequest request){
        ResultVo resultVo = null;
        String loginpwd = user.getLoginpwd();
        setLoginPwd(user);
        UserVo userVo = userMapper.getUserByLoginname(user.getLoginname());

        if(userVo!=null && userVo.getUserid()>0){
            user.setUserid(userVo.getUserid());
            user.setUsername(userVo.getUsername());
            user.setSchoolid(userVo.getSchoolid());
            if (!userVo.getLoginpwd().equals(user.getLoginpwd())) {
                resultVo = new ResultVo(1, "密码错误");
            } else {
                if (userVo.getUsertype() == 999) {
                    resultVo = new ResultVo(0, "登录成功", userVo);
                } else if (userVo.getUsertype() == 2) {
                    resultVo = new ResultVo(1, "禁止学生登录");
                } else if(userVo.getSchoolstate()!=1){
                    resultVo = new ResultVo(1,"用户所在学校暂无权限");
                }else{
                    if(userVo.getUserstate()!=1){
                        resultVo = new ResultVo(1,"该账户无权访问，请联系老师");
                    }else{
                        resultVo = new ResultVo(0,"登录成功",userVo);
                    }
                }
            }
        }else{
            resultVo = new ResultVo(1,"该账号不存在");
        }

        return getLogResultVo(user, resultVo, loginpwd, request);
    }

    //教师登录
    public ResultVo loginTeacher(TUser user,HttpServletRequest request){
        ResultVo resultVo = null;
        String loginpwd = user.getLoginpwd();
        setLoginPwd(user);
        UserVo userVo = userMapper.getUserByLoginname(user.getLoginname());

        if(userVo!=null && userVo.getUserid()>0) {
            user.setUserid(userVo.getUserid());
            user.setUsername(userVo.getUsername());
            user.setSchoolid(userVo.getSchoolid());
            if (!userVo.getLoginpwd().equals(user.getLoginpwd())) {
                resultVo = new ResultVo(1, "密码错误");
            } else {
                if (userVo.getUsertype() != 1) {
                    resultVo = new ResultVo(1, "禁止非教师账号登录");
                } else if (userVo.getSchoolstate() != 1) {
                    resultVo = new ResultVo(1, "用户所在学校暂无权限");
                }else{
                    if(userVo.getUserstate()!=1){
                        resultVo = new ResultVo(1,"该账户无权访问，请联系老师");
                    }else{
                        resultVo = new ResultVo(0,"登录成功",userVo);
                    }
                }
            }
        }else{
            resultVo = new ResultVo(1,"该账号不存在");
        }

        return getLogResultVo(user, resultVo, loginpwd, request);
    }

    /**
     * 学生登录
     * @Param [user]
     * @Author ywx
     * @Date 2020/6/2 11:14
     * @return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo loginStudent(TUser user, HttpServletRequest request){
        ResultVo resultVo = null;
        String loginpwd = user.getLoginpwd();
        setLoginPwd(user);
        String mac = user.getMac();
        UserVo userVo = userMapper.getUserByLoginnameMac(user.getLoginname(),mac);

        if(userVo!=null && userVo.getUserid()>0) {
            Integer sbindmac = userVo.getSbindmac();
            user.setUserid(userVo.getUserid());
            user.setUsername(userVo.getUsername());
            user.setSchoolid(userVo.getSchoolid());
            if (!userVo.getLoginpwd().equals(user.getLoginpwd())) {
                resultVo = new ResultVo(1, "密码错误");
            } else {
                if (userVo.getUsertype() != 2) {
                    resultVo = new ResultVo(1, "登录失败，仅允许学生登录");
                } else if (userVo.getXznum() == 0) {
                    resultVo = new ResultVo(1,"该账号未分配班级");
                } else if (sbindmac == 1 && N_Utils.isNotEmpty(mac)) {
                    if (N_Utils.isNotEmpty(userVo.getMac()) && !mac.equals(userVo.getMac())) {
                        resultVo = new ResultVo(1, "登录失败，该账号已与其他设备关联");
                    } else {
                        //20211221添加注释，sql语句修改了，不绑定的情况下会出现一个mac可以绑定多个帐号，绑定情况下没问题
                        if (N_Utils.isNotEmpty(userVo.getMacname()) && !userVo.getMacname().equals(user.getLoginname())) {
                            resultVo = new ResultVo(1, "登录失败，该设备已与其他账号关联");
                        } else {
                            resultVo = new ResultVo(0, "登录成功", userVo);
                        }
                    }
                } else {
                    resultVo = new ResultVo(0, "登录成功", userVo);
                }
                //更新学生硬件绑定信息
                userVo.setMac(mac);
                userVo.setBindmac(sbindmac);
                userMapper.updateBindMac(userVo);
            }
        }else{
            resultVo = new ResultVo(1,"该账号不存在");
        }

        return getLogResultVo(user, resultVo, loginpwd, request);
    }

    /**
     * 登录日志
     * @Param [loginname, device, user, result]
     * @Author ywx
     * @Date 2020/6/2 11:08
     * @return com.xhkjedu.vo.ResultVo
     **/
    private ResultVo getLogResultVo(TUser user, ResultVo result, String loginpwd, HttpServletRequest request) {
        String loginname = user.getLoginname();
        TUserLog userLog = new TUserLog();
        userLog.setLoginname(loginname);
        if (result.getCode() == 0) {
            UserVo userVo = (UserVo) result.getObj();
            String usertime = userVo.getUsertime();
            if (N_Utils.isNotEmpty(usertime)) {
                String dayBegin = DatesUtil.getDayBegin();
                if (usertime.compareTo(dayBegin) < 0) {
                    result.setCode(1);
                    result.setMsg("账号已过有效期，请联系管理员");
                    return result;
                }
            }

            //redis
            String ticketKey = UUID.randomUUID().toString().replace("-", "");
            String token = DESUtils.encrypt(ticketKey, ConfigKey.secretKey);//加密
            String key = userVo.getCloudcode()+"_"+userVo.getUserid()+"_"+user.getUltype();
            Boolean redis_rtn = set(key, token, ConfigKey.redisdatatime);
            if(!redis_rtn){
                result.setCode(1);
                result.setMsg("加入缓存出错!");
                return result;
            } else {
                userLog.setLoginstate(1);
                userLog.setLogtxt("登录成功");
                userVo.setToken_key(key);
                userVo.setToken_value(token);
                result.setObj(userVo);
            }
        } else {
            userLog.setLoginstate(2);
            userLog.setLogtxt(result.getMsg()+"，密码："+loginpwd);
        }
        try {
            userLog.setUserid(user.getUserid());
            userLog.setUsername(user.getUsername());
            userLog.setSchoolid(user.getSchoolid());
            userLog.setLogip(user.getLogip());
            userLog.setLogaddress(user.getLogaddress());
            userLog.setUltype(user.getUltype());
            userLog.setVersionnum(user.getVersionnum());
            userLog.setLogtime(N_Utils.getSecondTimestamp());
            userLogMapper.insertSelective(userLog);
        } catch (Exception e) {
            log.error("添加登录日志失败:" + e.getMessage());
            result = new ResultVo(1, "保存失败");
        }
        return result;
    }

    /**
     * 添加用户信息
     * @Param [user]
     * @Author ywx
     * @Date 2020/6/2 9:45
     * @return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo save(TUser user) {
        Integer schoolid = user.getSchoolid();
        Integer usernum = schoolMapper.getUserNumById(schoolid);
        if (usernum != null) {
            Integer usernum2 = userMapper.getUserNumBySchoolId(schoolid);
            usernum2 = usernum2 + 1;//学校账号数量+本次添加的人
            if (usernum.compareTo(usernum2) == -1) {
                return new ResultVo(1,"账号数量已超限，请联系管理员");
            }
        }
        user.setUserstate(1);
        user.setCreatetime(N_Utils.getSecondTimestamp());
        setLoginPwd(user);
        userMapper.insertUseGeneratedKeys(user);
        return new ResultVo(0, "保存成功",user.getId());
    }

    /**
     * 修改用户信息
     * @Param [user]
     * @Author ywx
     * @Date 2020/6/2 9:51
     * @return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo update(TUser user) {
        if (N_Utils.isNotEmpty(user.getLoginpwd())) {
            setLoginPwd(user);
        }
        String loginname = user.getLoginname();
        if (N_Utils.isNotEmpty(loginname)) {
            TUser user1 = getObjByLoginName(loginname);
            //登录账号改变并且和别的账号冲突
            if (user1!=null && !user1.getUserid().equals(user.getUserid())) {
                return new ResultVo(1,"此用户已存在，请更换！");
            }
        }
        user.setCreatetime(N_Utils.getSecondTimestamp());
        userMapper.updateByPrimaryKeySelective(user);
        return new ResultVo(0,"修改成功");
    }

    private void setLoginPwd(TUser user) {
        MD5 m = new MD5();
        String loginpwd = m.getMD5ofStr(m.getMD5ofStr(m.getMD5ofStr(user.getLoginpwd())));
        user.setLoginpwd(loginpwd);
    }

    /**
     * 删除用户
     * @Param [user]
     * @Author ywx
     * @Date 2020/6/2 9:59
     * @return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo deleteById(TUser user) {
        user.setDeletetime(N_Utils.getSecondTimestamp());
        userMapper.deleteById(user);
        return new ResultVo(0,"删除成功");
    }

    /**
     * 获取用户列表
     * @Param [user]
     * @Author ywx
     * @Date 2020/6/2 10:06
     * @return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo findAll(TUser user) {
        PageHelper.startPage(user.getPage(), user.getPageSize());
        List<UserVo> list = userMapper.findAll(user);
        PageResult pageResult = PageUtil.getPageResult(new PageInfo<>(list));
        return new ResultVo(0,"获取列表成功", pageResult);
    }

    /**
     * 获取用户详情
     * @Param [userid]
     * @Author ywx
     * @Date 2020/6/2 10:23
     * @return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo findById(Integer userid) {
        UserVo userVo = userMapper.findById(userid);
        return new ResultVo(0,"获取详情成功", userVo);
    }

    /**
     * 修改用户密码
     * @Param [user]
     * @Author ywx
     * @Date 2020/6/2 10:30
     * @return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo updatePwd(TUser user) {
        UserVo userVo = userMapper.findById(user.getUserid());

        String oldpwd = this.md5pwd(user.getOldpwd());
        if(userVo.getLoginpwd().equals(oldpwd)){
            setLoginPwd(user);
            userMapper.updatePwd(user);
            return new ResultVo(0,"修改成功");
        }else{
            return new ResultVo(1,"原密码错误");
        }

    }

    /**
     * 修改用户头像
     * @Param [user]
     * @Author ywx
     * @Date 2020/6/2 10:37
     * @return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo updateHeadPic(TUser user) {
        userMapper.updateHeadPic(user);
        return new ResultVo(0,"修改成功");
    }

    /**
     * 检查登录账号是否存在
     * @Param [loginname]
     * @Author ywx
     * @Date 2020/6/2 10:44
     * @return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo checkLoginName(String loginname) {
        Integer num = userMapper.checkLoginName(loginname);
        if (num != 0){
            return new ResultVo(1,"此用户已存在，请更换！");
        } else {
            return new ResultVo(0, "检查登录账号是否存在成功");
        }
    }

    /**
     * 根据登录账号获取用户详情
     * @Param [loginname]
     * @Author ywx
     * @Date 2020/6/3 9:33
     * @return com.xhkjedu.model.system.TUser
     **/
    public TUser getObjByLoginName(String loginname) {
        return userMapper.getObjByLoginName(loginname);
    }

    /**
     * 批量添加
     * @Param [users]
     * @Author ywx
     * @Date 2020/6/3 18:27
     * @return void
     **/
    public void batchSave(List<TUser> users) throws Exception {
        Integer schoolid = users.get(0).getSchoolid();
        Integer usernum = schoolMapper.getUserNumById(schoolid);
        if (usernum != null) {
            Integer usernum2 = userMapper.getUserNumBySchoolId(schoolid);
            usernum2 = usernum2 + users.size();//学校账号数量+本次添加的人
            if (usernum.compareTo(usernum2) == -1) {
                throw new Exception("账号数量已超限，请联系管理员");
            }
        }
        userMapper.insertList(users);
    }

    /**
     * 修改用户状态
     * @Param [userid, userstate]
     * @Author ywx
     * @Date 2020/6/4 9:08
     * @return void
     **/
    public void updateState(Integer userid, int userstate) {
        userMapper.updateState(userid, userstate);
    }

    /**
     * 导入教师
     * @Param [file, tUser]
     * @Author ywx
     * @Date 2020/6/5 8:47
     * @return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo importTeacher(MultipartFile file, TUser tUser) {
        Integer schoolid = tUser.getSchoolid();
        Integer createid = tUser.getCreateid();
        N_Utils.validation(new Object[]{schoolid,"学校id",1,createid,"创建人id",1});
        try {
            Map<String,Object> map = PoiUtils.readExcelUser(file, 1);
            List<TUser> teachers = (List<TUser>)map.get("users");
            map.remove("users");//导入成功的教师信息不返回给前端

            List<TUser> users = new ArrayList<>();
            // 以下教师信息已存在
            List<TUser> usersExisted = new ArrayList<>();
            for (TUser user : teachers) {
                TUser u = getObjByLoginName(user.getLoginname());
                if (null == u){
                    user.setSchoolid(schoolid);
                    user.setCreateid(createid);
                    user.setCreatetime(N_Utils.getSecondTimestamp());
                    user.setUserstate(1);
                    users.add(user);
                } else {
                    if (1 != u.getUsertype() || schoolid != u.getSchoolid()){
                        usersExisted.add(u);
                    } else {
                        usersExisted.add(u);
                    }
                }
            }

            //保存用户信息
            if (!CollectionUtils.isEmpty(users)) {
                batchSave(users);
            }
            map.put("usersExisted",usersExisted);
            if (N_Utils.isListNotEmpty(usersExisted)){
                map.put("code",1);
            } else {
                map.put("code",0);
            }
            return new ResultVo(0, "导入成功", map);
        } catch (Exception e) {
            log.error("导入失败 : " + e.getMessage());
            return new ResultVo(1, "导入失败 : " + e.getMessage());
        }
    }

    private String md5pwd(String pwd){
        MD5 m = new MD5();
        String loginpwd = m.getMD5ofStr(m.getMD5ofStr(m.getMD5ofStr(pwd)));
        return loginpwd;
    }

    /**
     *功能描述 修改操作密码
     * @author WN
     * @date 2020/6/17
     * @param  * @param user
     * @return void
     */
    public void updateOpepasswordByUserId(TUser user){
        String pwd = md5pwd(user.getOpepassword());
        userMapper.updateOpepasswordByUserId(user.getUserid(),pwd);
    }

    /**
     *功能描述 验证用户操作密码
     * @author WN
     * @date 2020/6/17
     * @param  * @param user
     * @return java.lang.Integer
     */
    public Integer getUserIdByOpepassword(TUser user){
        String pwd = md5pwd(user.getOpepassword());
        return userMapper.getUserIdByOpepassword(user.getUserid(),pwd);
    }

    /**
     * 教师基本信息
     * @Param [userid]
     * @Author ywx
     * @Date 2020/6/29 16:23
     * @return com.xhkjedu.suser.vo.system.TeacherVo
     **/
    public TeacherVo getTeacherInfo(Integer userid, Integer classtype) {
        return userMapper.getTeacherInfo(userid, classtype);
    }

    /**
     * 重置密码
     * @Param [user]
     * @Author ywx
     * @Date 2020/7/30 14:25
     * @return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo resetPwd(TUser user) {
        setLoginPwd(user);
        userMapper.updatePwd(user);
        return new ResultVo(0,"重置密码成功");
    }

    /**
     * 解除mac绑定
     * @Param [studentid]
     * @Author ywx
     * @Date 2020/9/18 11:56
     * @return void
     **/
    public void removeBind(Integer studentid) {
        UserVo user = new UserVo();
        user.setUserid(studentid);
        user.setBindmac(0);
        user.setMac(null);
        userMapper.updateBindMac(user);
    }

    /**
     * 生成修改码
     * @Param [userphone]
     * @Author ywx
     * @Date 2020/10/13 15:08
     * @return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo updateCode(String userphone) {
        String templateCode = "xgmm";
        if (generateCode(userphone, templateCode)) return new ResultVo(1, "请输入验证码或者在5分钟之后重新点击");
        return new ResultVo(0,"生成修改码成功");
    }

    /**
     * 生成验证码
     * @Param [userphone, templateCode]
     * @Author ywx
     * @Date 2020/5/8 10:04
     * @return boolean
     **/
    private boolean generateCode(String userphone, String templateCode) {
        Integer msgNum = msgMapper.getMsgNumByUserPhone(userphone);//5分钟内发送短信次数
        if (msgNum != 0) return true;

        Integer messageCode = N_Utils.messageCode();
        String msgtxt = "";
        if ("yhzc".equals(templateCode)) {//用户注册验证码
            msgtxt += "验证码" + messageCode + "，您正在注册成为新用户，感谢您的支持！";
        } else if ("xgmm".equals(templateCode)) {//修改密码验证码
            msgtxt += "验证码" + messageCode + "，您正在尝试修改登录密码，请妥善保管账户信息。";
        } else if ("dlqr".equals(templateCode)) {//登录确认验证码
            msgtxt += "验证码" + messageCode + "，您正在登录，若非本人操作，请勿泄露。";
        }
        ZtMsgVo m = new ZtMsgVo();
        m.setMobile(userphone);
        m.setContent(msgtxt);
        SMSSender.sendSms(m);

        //添加短信信息
        TMsg msg = new TMsg();
        msg.setUserphone(userphone);
        msg.setMsgcode(messageCode);
        msg.setMsgtxt(msgtxt);
        msg.setCreatetime(N_Utils.getSecondTimestamp());
        msgMapper.insertSelective(msg);
        return false;
    }

    /**
     * 通过短信修改密码
     * @Param [user]
     * @Author ywx
     * @Date 2020/10/13 15:17
     * @return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo updatePwdByCode(TUser user) {
        setLoginPwd(user);
        ResultVo x = getCodeResultVo(user.getUserphone(), user.getMsgcode());
        if (x != null) return x;
        userMapper.updatePwdByCode(user);
        return new ResultVo(0,"修改成功");
    }

    /**
     * 判断验证码是否有效
     * @Param [userphone, messageCode]
     * @Author ywx
     * @Date 2020/5/8 10:21
     * @return com.xhkjedu.model.baseinfo.ResultVo
     **/
    private ResultVo getCodeResultVo(String userphone, Integer messageCode) {
        TMsg msg = msgMapper.findByUserPhoneAndCode(userphone, messageCode);//根据手机号和验证码获取短信信息
        if (msg == null) return new ResultVo(1, "验证码有误");
        int secondTimestamp = N_Utils.getSecondTimestamp() - 300;//验证码有效时间为5分钟
        if (msg.getCreatetime() < secondTimestamp) return new ResultVo(1, "验证码失效");
        return null;
    }

    /**
     * 生成登录确认码
     * @Param [userphone]
     * @Author ywx
     * @Date 2020/10/13 15:19
     * @return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo loginCode(String userphone) {
        N_Utils.validation(new Object[]{userphone,"手机号",2});
        String templateCode = "dlqr";
        if (generateCode(userphone, templateCode)) return new ResultVo(1, "请输入验证码或者在5分钟之后重新点击");
        return new ResultVo(0,"生成登录确认码成功");
    }

    /**
     * 验证码登录
     * @Param [user]
     * @Author ywx
     * @Date 2020/10/13 15:36
     * @return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo loginByCode(TUser user,HttpServletRequest request) {
        ResultVo resultVo = getCodeResultVo(user.getUserphone(), user.getMsgcode());
        if (resultVo != null) return resultVo;
        UserVo userVo = userMapper.getUserByLoginname(user.getUserphone());

        if(userVo!=null && userVo.getUserid()>0){
            user.setUserid(userVo.getUserid());
            user.setUsername(userVo.getUsername());
            user.setSchoolid(userVo.getSchoolid());
            if(userVo.getUsertype()==999){
                resultVo = new ResultVo(0,"登录成功",userVo);
            } else if(userVo.getUsertype()==2){
                resultVo = new ResultVo(1,"禁止学生登录");
            } else if(userVo.getSchoolstate()!=1){
                resultVo = new ResultVo(1,"用户所在学校暂无权限");
            }else{
                if(userVo.getUserstate()!=1){
                    resultVo = new ResultVo(1,"该账户无权访问，请联系老师");
                }else{
                    resultVo = new ResultVo(0,"登录成功",userVo);
                }
            }
        }else{
            resultVo = new ResultVo(1,"该账号不存在");
        }

        return getLogResultVo(user, resultVo, user.getMsgcode()+"", request);
    }

    public ResultVo batchUpdatePwd(TUser user) {
        setLoginPwd(user);
        userMapper.batchUpdatePwd(user);
        return new ResultVo(0,"修改成功");
    }

    public ResultVo checkLogin(Integer userid, String loginpwd) {
        Map map = new HashMap();
        boolean needLogin = false;
        boolean needPhone = false;
        TUser user = userMapper.selectByPrimaryKey(userid);
        String usertime = user.getUsertime();
        String dayBegin = DatesUtil.getDayBegin();
        String msg = "用户信息正常";
        if (!loginpwd.equals(user.getLoginpwd())) {
            needLogin = true;
            msg = "密码已修改，请重新登录";
        }
        if (!needLogin && N_Utils.isNotEmpty(usertime) && usertime.compareTo(dayBegin) < 0) {
            needLogin = true;
            msg = "账号已过有效期，请联系管理员";
        }
        if (user.getUsertype() == 2) {
            needPhone = userMapper.checkUserPhone(userid);
            if (!needLogin && needPhone) msg = "未绑定手机号";
        }
        map.put("needLogin", needLogin);
        map.put("needPhone", needPhone);
        return new ResultVo(0, msg, map);
    }

    //用户选择列表
    public List<Map> listForSel(AssessParams user) {
        Integer year = schoolMapper.getYearById(user.getSchoolid());
        return userMapper.listForSel(user, year);
    }

    //用户名登录
    public ResultVo loginByName(TUser user, HttpServletRequest request) {
        ResultVo resultVo = null;
        UserVo userVo = userMapper.getUserByLoginname(user.getLoginname());

        if(userVo!=null && userVo.getUserid()>0){
            user.setUserid(userVo.getUserid());
            user.setUsername(userVo.getUsername());
            user.setSchoolid(userVo.getSchoolid());
            user.setUltype("p_web_t");
            if(userVo.getUsertype()==999){
                resultVo = new ResultVo(0,"登录成功",userVo);
            } else if(userVo.getUsertype()==2){
                resultVo = new ResultVo(1,"禁止学生登录");
            } else if(userVo.getSchoolstate()!=1){
                resultVo = new ResultVo(1,"用户所在学校暂无权限");
            }else{
                if(userVo.getUserstate()!=1){
                    resultVo = new ResultVo(1,"该账户无权访问，请联系老师");
                }else{
                    resultVo = new ResultVo(0,"登录成功",userVo);
                }
            }
        }else{
            resultVo = new ResultVo(1,"该账号不存在");
        }

        return getLogResultVo(user, resultVo, "", request);
    }

    /**
     * 获取头像是否可以修改
     * @Param [userid]
     * @Author ywx
     * @Date 2022/3/14 9:58
     * @return java.lang.Integer
     **/
    public Integer getHeadTime(Integer userid) {
        return userMapper.getHeadTime(userid);
    }
}
