package com.xhkjedu.suser.utils;

import com.xhkjedu.suser.model.system.TUser;
import com.xhkjedu.utils.MD5;
import com.xhkjedu.utils.N_Utils;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author ywx
 * @classname PoiUtils
 * @description
 * @date 2020/6/3 16:33
 **/
@Component
@Slf4j
public class PoiUtils {
    //2003- 版本的excel
    private final static String excel2003L = ".xls";
    //2007+ 版本的excel
    private final static String excel2007U = ".xlsx";

    /**
     * 导入班级学生
     *
     * @return java.lang.String
     * @Param [file]
     * @Author ywx
     * @Date 2020/6/3 16:43
     **/
    public static Map<String, Object> readExcelUser(MultipartFile file, Integer usertype) throws Exception {
        Map<String, Object> resultMap = new HashMap<>();
        try {
            // 数据校验
            isExcelEmpty(file);
            InputStream in = file.getInputStream();
            String fileName = file.getOriginalFilename();
            Workbook work = getWorkbook(in, fileName);
            Sheet sheet;// 分表数据
            Row row;// 行数据
            sheet = work.getSheetAt(0);//只取第一个sheet
            if (sheet == null) {
                throw new Exception("Excel第一个分表内容为空！");
            }

            int rows = sheet.getLastRowNum(); // 获取最后一个实际行的下标，比行数小1
            int firstRowNum = sheet.getFirstRowNum(); // 获取第一个实际行的下标，（firstRowNum=0）

            // 导入用户信息
            List<TUser> users = new ArrayList<>();
            // excel已存在学生信息
//            List<TUser> excelExisted = new ArrayList<>();
            Map<String,String> nameMap = new HashMap<>();

            /*
             * 遍历当前sheet中的所有行
             * 排除第一行  第一行为标题头 不纳入遍历
             */
            for (int x = firstRowNum+1; x <= rows; x++) {
                // 获取行
                row = sheet.getRow(x);
                if (row == null ){//若此行为空，则跳出循环继续下一行
                    continue;
                } else if (isEmptyCell(row.getCell(0)) && isEmptyCell(row.getCell(1))){
                    continue;
                }
                TUser user = new TUser();// 每一行一个学生信息
                user.setUsertype(usertype);
                int firstCellNum = row.getFirstCellNum();// 每一行的第一列，（firstCellNum=0）
                if (firstCellNum == 1) throw new Exception("第" + (x + 1) + "行姓名不能为空");
                int lastCellNum = row.getLastCellNum();// 每一行的最后一列，获取列数，比最后一列列标大1
                //遍历所有的列
                for (int y = firstCellNum; y < lastCellNum; y++) {
                    Cell cell = row.getCell(y);
                    if (y == 0){   // 姓名
                        setUserName(cell, x, user);
                    } else if(y == 1){// 登录名
                        if (isEmptyCell(cell) ){
                            throw new Exception("第"+(x+1)+"行登录账号不能为空");
                        } else{
                            String loginname = getCellValue(cell);
                            user.setLoginname(loginname);//登录名默认为学号
                            if (2 == usertype) {//导入学生登录账号默认为学号
                                user.setStudentno(loginname);
                            }
                            MD5 md5 = new MD5();
                            user.setLoginpwd(md5.getMD5ofStr(md5.getMD5ofStr(md5.getMD5ofStr("123456"))));
                        }
                    } else if(y == 2){//性别
                        if (isEmptyCell(cell) ){
                            user.setUsersex(0);
                        } else {
                            String usersex = getCellValue(cell);
                            if (usersex.equals("女")){
                                user.setUsersex(2);
                            } else {
                                user.setUsersex(1);
                            }
                        }
                    } else if(y == 3){//手机号
                        setUserPhone(cell, x, user);
                    } else if(y == 4){//出生日期
                        if (!isEmptyCell(cell) ){
                            String birthDay = getCellValue(cell);
                            user.setUserbirthday(N_Utils.getDataToTimestamp(birthDay,"yyyy-MM-dd"));
                        }
                    } else if(y == 5){//身份证号
                        if (isEmptyCell(cell) ){
                            user.setCardid("");
                        } else {
                            user.setCardid(getCellValue(cell));
                        }
                    }
                }

                String loginname = user.getLoginname();
                if (N_Utils.isNotEmpty(nameMap.get(loginname))) {
//                    excelExisted.add(user);
                } else {
                    nameMap.put(loginname,loginname);
                    users.add(user);
                }
            }

            resultMap.put("code" , 0);
            resultMap.put("users" , users);
//            resultMap.put("excelExisted" , excelExisted);
        }catch (Exception e){
            resultMap.put("code" , 1);
            resultMap.put("msg" , e.getMessage());
            throw new Exception(e.getMessage());
        }
        return resultMap;
    }

    private static void setUserName(Cell cell, int x, TUser user) throws Exception {
        if (isEmptyCell(cell)) {
            throw new Exception("第" + (x + 1) + "行姓名不能为空");
        } else {
            user.setUsername(getCellValue(cell));
        }
    }

    private static void setUserPhone(Cell cell, int x, TUser user) throws Exception {
        if (isEmptyCell(cell)) {
            user.setUserphone("");
        } else {
            String phone = getCellValue(cell);
            if (!N_Utils.isPhone(phone)) {
                throw new Exception("第" + (x + 1) + "行手机号不合法");
            }
            user.setUserphone(phone);
        }
    }

    /**
     * 根据文件后缀，自适应上传文件的版本
     * @Param [inStr, fileName]
     * @Author ywx
     * @Date 2020/6/3 16:42
     * @return org.apache.poi.ss.usermodel.Workbook
     **/
    public static Workbook getWorkbook(InputStream inStr, String fileName) throws Exception{
        Workbook wb;
        String fileType = fileName.substring(fileName.lastIndexOf("."));
        if(excel2003L.equals(fileType)){
            wb = new HSSFWorkbook(inStr);  //2003-
        }else if(excel2007U.equals(fileType)){
            wb = new XSSFWorkbook(inStr);  //2007+
        }else{
            throw new Exception("Excel格式解析有误！");
        }
        return wb;
    }

    /**
     * 判断单元格是否是空值
     * @Param [cell]
     * @Author ywx
     * @Date 2020/6/3 16:37
     * @return java.lang.Boolean
     **/
    public static Boolean isEmptyCell(Cell cell){
        if(cell != null){
            Boolean rtn = true;
            if(N_Utils.isNotEmpty(PoiUtils.getCellValue(cell))){
                rtn = false;
            }
            return rtn;
        } else {
            return true;
        }
    }

    /**
     * 对表格中数值进行格式化
     * @Param [cell]
     * @Author ywx
     * @Date 2020/6/3 16:41
     * @return java.lang.Object
     **/
    public static String getCellValue(Cell cell){
        String value = null;
        DecimalFormat df = new DecimalFormat("0");  //格式化number String字符
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");  //日期格式化
        DecimalFormat df2 = new DecimalFormat("0");  //格式化数字

        switch (cell.getCellType()) {
            case Cell.CELL_TYPE_STRING:
                value = cell.getRichStringCellValue().getString();
                break;
            case Cell.CELL_TYPE_NUMERIC:
                String cellStyle = cell.getCellStyle().getDataFormatString();
                if("General".equals(cellStyle)){
                    value = df.format(cell.getNumericCellValue());
                }else if("m/d/yy".equals(cellStyle)){
                    value = sdf.format(cell.getDateCellValue());
                }else{
                    value = df2.format(cell.getNumericCellValue());
                }
                break;
            case Cell.CELL_TYPE_BOOLEAN:
                boolean rtn = cell.getBooleanCellValue();
                if (rtn){
                    value = "1";//真
                } else {
                    value = "0";//假
                }
                break;
            case Cell.CELL_TYPE_BLANK:
                value = "";
                break;
            case HSSFCell.CELL_TYPE_FORMULA://读取excel公式的值
                try {
                    value = String.valueOf(cell.getNumericCellValue());
                } catch (IllegalStateException e) {
                    value = String.valueOf(cell.getRichStringCellValue());
                }
                break;
            default:
                break;
        }
        return N_Utils.strTrim(value);
    }

    /**
     * 检验Excel内容是否为空
     * @Param [file]
     * @Author ywx
     * @Date 2020/6/3 16:36
     * @return void
     **/
    public static void isExcelEmpty(MultipartFile file) throws Exception {
        if (null == file || file.isEmpty()) {
            throw new Exception("文件不存在！");
        }
        InputStream in = file.getInputStream();
        String fileName = file.getOriginalFilename();
        Workbook work = getWorkbook(in, fileName);
        if (null == work) {
            throw new Exception("Excel内容为空！");
        }
        Integer sheet_size = work.getNumberOfSheets();
        if (sheet_size == 0) {
            throw new Exception("Excel内容为空！");
        }
    }
}
